@@ -36,7 +36,7 @@ def _pty_size():
     """
     Obtain (rows, cols) tuple for sizing a pty on the remote end.
 
-    Defaults to 80x24 (which is also the Paramiko default) but will detect
+    Defaults to 80x24 (which is also the 'ssh' lib's default) but will detect
     local (stdout-based) terminal window size on non-Windows platforms.
     """
     rows, cols = 24, 80
@@ -248,7 +248,7 @@ def prompt(text, key=None, default='', validate=None):
             prompt('I seriously need an answer on this! ')
 
     """
-    handle_prompt_abort()
+    handle_prompt_abort("a user-specified prompt() call")
     # Store previous env value for later display, if necessary
     if key:
         previous_value = env.get(key)
